@@ -1,7 +1,7 @@
 // ----------------------------------------------------------------------------
 // cw.h  --  morse code modem
 //
-// Copyright (C) 2006
+// Copyright (C) 2006-2009
 //		Dave Freese, W1HKJ
 //
 // This file is part of fldigi.  Adapted in part from code contained in 
@@ -11,19 +11,18 @@
 //  Copyright (C) 2004
 //  Lawrence Glaister (ve7it@shaw.ca)
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef _CW_H
@@ -84,7 +83,6 @@ enum CW_EVENT {
 	CW_QUERY_EVENT
 };
 
-//class cw : public morse, public modem {
 class cw : public modem {
 protected:
 	int				symbollen;		// length of a dot in sound samples (tx)
@@ -100,6 +98,7 @@ protected:
 	int bitfilterlen;
 
 	CW_RX_STATE		cw_receive_state;	// Indicates receive state 
+	CW_RX_STATE		old_cw_receive_state;
 	CW_EVENT		cw_event;			// functions used by cw process routine 
 	 
 	double pipe[CWMaxSymLen];			// storage for sync scope data
