@@ -5,57 +5,40 @@
 // Copyright (C) 2006-2007
 //              Dave Freese, W1HKJ
 //
-// Copyright (C) 2007-2008
+// Copyright (C) 2007-2009
 //              Stelios Bounanos, M0GLD
 //
 // This file is part of fldigi.
 //
-// fldigi is free software; you can redistribute it and/or modify
+// Fldigi is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
 //
-// fldigi is distributed in the hope that it will be useful,
+// Fldigi is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with fldigi; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// along with fldigi.  If not, see <http://www.gnu.org/licenses/>.
 // ----------------------------------------------------------------------------
 
 #ifndef _SOUND_H
 #define _SOUND_H
 
-#include <cstring>
 #include <string>
-#include <vector>
+#include <cstring>
+#include <climits>
+
 #if USE_SNDFILE
-	#include <sndfile.h>
-#endif
-#if USE_PORTAUDIO
-	#include <portaudio.h>
-#endif
-#if USE_PULSEAUDIO
-	#include <pulse/simple.h>
-	#include <pulse/error.h>
-extern "C" { const char* pa_get_library_version(void); };
+#  include <sndfile.h>
 #endif
+
 #include <samplerate.h>
-#include <semaphore.h>
-#include <limits.h>
 
-#include "ringbuffer.h"
-
-#define MAXSC 32767.0;
-#define maxsc 32000.0
-//#define maxsc 16384.0
 #define SCBLOCKSIZE 512
 
-#define	SND_BUF_LEN		65536
-//#define	SRC_BUF_LEN		(8*SND_BUF_LEN)
-
 
 class SndException : public std::exception
 {
@@ -79,32 +62,6 @@ protected:
 	std::string	msg;
 };
 
-#if USE_PORTAUDIO
-class SndPortException : public SndException
-{
-public:
-	SndPortException(int err_ = 0)
-		: SndException(err_, std::string("PortAudio error: ") + err_to_str(err_))
-	{ }
-	SndPortException(const char* msg_) : SndException(msg_) { }
-protected:
-	const char* err_to_str(int e) { return Pa_GetErrorText(e); }
-};
-#endif
-
-#if USE_PULSEAUDIO
-class SndPulseException : public SndException
-{
-public:
-	SndPulseException(int err_ = 0)
-		: SndException(err_, std::string("PulseAudio error: ") + err_to_str(err_))
-	{ }
-	SndPulseException(const char* msg_) : SndException(msg_) { }
-protected:
-	const char* err_to_str(int e) { return pa_strerror(e); }
-};
-#endif
-
 
 class SoundBase {
 protected:
@@ -112,20 +69,17 @@ protected:
 	int		txppm;
 	int		rxppm;
 
-// for interface to the samplerate resampling library
+	// for interface to the samplerate resampling library
 	SRC_STATE	*tx_src_state;
-	SRC_DATA	*tx_src_data;
 	SRC_STATE	*rx_src_state;
-	SRC_DATA	*rx_src_data;
-	float		*snd_buffer;
-	float		*src_buffer;
 	double		*wrt_buffer;
 
 #if USE_SNDFILE
 	SNDFILE* ofCapture;
 	SNDFILE* ifPlayback;
 	SNDFILE* ofGenerate;
-	sf_count_t  read_file(SNDFILE* file, double* buf, size_t count);
+	sf_count_t  read_file(SNDFILE* file, float* buf, size_t count);
+	sf_count_t  write_file(SNDFILE* file, float* buf, size_t count);
 	sf_count_t  write_file(SNDFILE* file, double* buf, size_t count);
 	bool	 format_supported(int format);
 	void	 tag_file(SNDFILE *sndfile, const char *title);
@@ -143,9 +97,9 @@ public:
 	virtual void    Abort(unsigned dir = UINT_MAX) = 0;
 	virtual size_t	Write(double *, size_t) = 0;
 	virtual size_t	Write_stereo(double *, double *, size_t) = 0;
-	virtual size_t	Read(double *, size_t) = 0;
+	virtual size_t	Read(float *, size_t) = 0;
 	virtual void    flush(unsigned dir = UINT_MAX) = 0;
-	virtual bool	must_close(void) = 0;
+	virtual bool	must_close(int dir = 0) = 0;
 #if USE_SNDFILE
 	void		get_file_params(const char* def_fname, const char** fname, int* format);
 	int		Capture(bool val);
@@ -170,6 +124,11 @@ private:
 	bool	formatok;
 	unsigned char	*cbuff;
 
+	SRC_DATA	*rx_src_data;
+	SRC_DATA	*tx_src_data;
+	float		*snd_buffer;
+	float		*src_buffer;
+
 	void	getVersion();
 	void	getCapabilities();
 	void	getFormats();
@@ -189,8 +148,8 @@ public:
 	void	Abort(unsigned dir = UINT_MAX) { Close(dir); }
 	size_t		Write(double *, size_t);
 	size_t		Write_stereo(double *, double *, size_t);
-	size_t		Read(double *, size_t);
-	bool		must_close(void) { return true; }
+	size_t		Read(float *, size_t);
+	bool		must_close(int dir = 0) { return true; }
 	void		flush(unsigned dir = UINT_MAX) { wait_till_finished(); }
 
 private:
@@ -208,8 +167,11 @@ private:
 
 
 #if USE_PORTAUDIO
-
-#include <pthread.h>
+#  include <pthread.h>
+#  include <semaphore.h>
+#  include <vector>
+#  include <portaudio.h>
+#  include "ringbuffer.h"
 
 class SoundPort : public SoundBase
 {
@@ -229,14 +191,15 @@ public:
 	void 		Abort(unsigned dir = UINT_MAX);
 	size_t 		Write(double *buf, size_t count);
 	size_t		Write_stereo(double *bufleft, double *bufright, size_t count);
-	size_t 		Read(double *buf, size_t count);
-	bool		must_close(void);
+	size_t 		Read(float *buf, size_t count);
+	bool		must_close(int dir = 0);
 	void		flush(unsigned dir = UINT_MAX);
 
 private:
         void		src_data_reset(unsigned dir);
         static long	src_read_cb(void* arg, float** data);
         size_t          resample_write(float* buf, size_t count);
+	device_iterator name_to_device(const std::string& name, unsigned dir);
         void 		init_stream(unsigned dir);
         void 		start_stream(unsigned dir);
         void 		pause_stream(unsigned dir);
@@ -250,11 +213,12 @@ private:
         static PaStreamFinishedCallback stream_stopped;
 
 private:
-        enum { STREAM_IN, STREAM_OUT };
         static bool                             pa_init;
 	static std::vector<const PaDeviceInfo*> devs;
         double	 				req_sample_rate;
         float* 					fbuf;
+	float*					src_buffer;
+	SRC_DATA	*tx_src_data;
 
         enum {
                 spa_continue = paContinue, spa_complete = paComplete,
@@ -281,9 +245,24 @@ private:
         } sd[2];
 };
 
+class SndPortException : public SndException
+{
+public:
+	SndPortException(int err_ = 0)
+		: SndException(err_, std::string("PortAudio error: ") + err_to_str(err_))
+	{ }
+	SndPortException(const char* msg_) : SndException(msg_) { }
+protected:
+	const char* err_to_str(int e) { return Pa_GetErrorText(e); }
+};
+
 #endif // USE_PORTAUDIO
 
+
 #if USE_PULSEAUDIO
+#  include <pulse/simple.h>
+#  include <pulse/error.h>
+extern "C" { const char* pa_get_library_version(void); };
 
 class SoundPulse : public SoundBase
 {
@@ -296,8 +275,8 @@ public:
 	void    Abort(unsigned dir = UINT_MAX);
 	size_t	Write(double* buf, size_t count);
 	size_t	Write_stereo(double* bufleft, double* bufright, size_t count);
-	size_t	Read(double *buf, size_t count);
-	bool	must_close(void) { return false; }
+	size_t	Read(float *buf, size_t count);
+	bool	must_close(int dir = 0) { return false; }
 	void	flush(unsigned dir = UINT_MAX);
 
 private:
@@ -309,11 +288,27 @@ private:
 	struct stream_data {
 		pa_simple*	stream;
 		pa_sample_spec	stream_params;
+		pa_buffer_attr  buffer_attrs;
 		pa_stream_direction_t dir;
 		double		src_ratio;
 		size_t		blocksize;
 	} sd[2];
+
+	SRC_DATA* tx_src_data;
 	float* fbuf;
+	float* snd_buffer;
+	float* src_buffer;
+};
+
+class SndPulseException : public SndException
+{
+public:
+	SndPulseException(int err_ = 0)
+		: SndException(err_, std::string("PulseAudio error: ") + err_to_str(err_))
+	{ }
+	SndPulseException(const char* msg_) : SndException(msg_) { }
+protected:
+	const char* err_to_str(int e) { return pa_strerror(e); }
 };
 
 #endif // USE_PULSEAUDIO
@@ -327,8 +322,8 @@ public:
 	void    Abort(unsigned) { }
 	size_t	Write(double* buf, size_t count);
 	size_t	Write_stereo(double* bufleft, double* bufright, size_t count);
-	size_t	Read(double *buf, size_t count);
-	bool	must_close(void) { return false; }
+	size_t	Read(float *buf, size_t count);
+	bool	must_close(int dir = 0) { return false; }
 	void	flush(unsigned) { }
 };
 
